@@ -1,6 +1,7 @@
 import collections.abc
 import copy
 import logging
+import sys
 import threading
 import typing
 import warnings
@@ -9,6 +10,7 @@ from typing import (
     TYPE_CHECKING,
     Any,
     Callable,
+    ClassVar,
     Dict,
     Iterable,
     List,
@@ -72,7 +74,7 @@ def _fill_cast(
     Convert to NumPy arrays. Some buffer objects do not get converted by forcecast.
     If not called by itself (inner=False), then will work through one level of tuple/list.
     """
-    if value is None or isinstance(value, (str, bytes)):
+    if value is None or isinstance(value, (str, bytes)):  # type: ignore[redundant-expr]
         return value  # type: ignore[return-value]
 
     if not inner and isinstance(value, (tuple, list)):
@@ -84,10 +86,23 @@ def _fill_cast(
     return value
 
 
+def mean_storage_sample_check(sample: Optional[ArrayLike]) -> None:
+    if sample is None:
+        raise TypeError("Sample key-argument (sample=) needs to be provided.")
+    seqs = (collections.abc.Sequence, np.ndarray)
+    msg1 = f"Sample key-argument needs to be a sequence, {sample.__class__.__name__} given."
+    if isinstance(sample, str) and not isinstance(sample, seqs):
+        raise ValueError(msg1)
+    sample_dim = np.array(sample).ndim
+    msg2 = f"Sample key-argument needs to be 1 dimensional, {sample_dim} given."
+    if sample_dim != 1:
+        raise ValueError(msg2)
+
+
 def _arg_shortcut(item: Union[Tuple[int, float, float], Axis, CppAxis]) -> CppAxis:
     if isinstance(item, tuple) and len(item) == 3:
-        msg = "Developer shortcut: will be removed in a future version"
-        warnings.warn(msg, FutureWarning)
+        msg = "Using () directly in constructor is a developer shortcut and will be removed in a future version"
+        warnings.warn(msg, FutureWarning, stacklevel=4)
         return _core.axis.regular_uoflow(item[0], item[1], item[2])  # type: ignore[return-value]
 
     if isinstance(item, Axis):
@@ -129,7 +144,10 @@ class Histogram:
     )
     # .metadata and ._variance_known are part of the dict
 
-    _family: object = boost_histogram
+    _family: ClassVar[object] = boost_histogram
+
+    axes: AxesTuple
+    _hist: CppHistogram
 
     def __init_subclass__(cls, *, family: Optional[object] = None) -> None:
         """
@@ -185,32 +203,34 @@ class Histogram:
 
         # Allow construction from a raw histogram object (internal)
         if len(axes) == 1 and isinstance(axes[0], tuple(_histograms)):
-            self._hist: Any = axes[0]
-            self.metadata = metadata
-            self.axes = self._generate_axes_()
+            cpp_hist: CppHistogram = axes[0]  # type: ignore[assignment]
+            self._from_histogram_cpp(cpp_hist)
+            if metadata:
+                self.metadata = metadata
             return
 
         # If we construct with another Histogram as the only positional argument,
         # support that too
         if len(axes) == 1 and isinstance(axes[0], Histogram):
-            # Special case - we can recursively call __init__ here
-            self.__init__(axes[0]._hist)  # type: ignore[misc] # pylint: disable=non-parent-init-called
-            self._from_histogram_object(axes[0])
+            normal_hist: Histogram = axes[0]
+            self._from_histogram_object(normal_hist)
+            if metadata:
+                self.metadata = metadata
             return
 
         # Support objects that provide a to_boost method, like Uproot
         if len(axes) == 1 and hasattr(axes[0], "_to_boost_histogram_"):
-            self.__init__(axes[0]._to_boost_histogram_())  # type: ignore[misc, union-attr] # pylint: disable=non-parent-init-called
+            self._from_histogram_object(axes[0]._to_boost_histogram_())  # type: ignore[union-attr]
             return
 
         if storage is None:
-            storage = Double()
+            storage = Double()  # type: ignore[unreachable]
 
         self.metadata = metadata
 
         # Check for missed parenthesis or incorrect types
         if not isinstance(storage, Storage):
-            msg_storage = (
+            msg_storage = (  # type: ignore[unreachable]
                 "Passing in an initialized storage has been removed. Please add ()."
             )
             msg_unknown = "Only storages allowed in storage argument"
@@ -226,12 +246,66 @@ class Histogram:
         # Check all available histograms, and if the storage matches, return that one
         for h in _histograms:
             if isinstance(storage, h._storage_type):
-                self._hist = h(axes, storage)
+                self._hist = h(axes, storage)  # type: ignore[unreachable]
                 self.axes = self._generate_axes_()
                 return
 
         raise TypeError("Unsupported storage")
 
+    @classmethod
+    def _clone(
+        cls: Type[H],
+        _hist: "Histogram | CppHistogram",
+        *,
+        other: "Histogram | None" = None,
+        memo: Any = NOTHING,
+    ) -> H:
+        """
+        Clone a histogram (possibly of a different base). Does not trigger __init__.
+        This will copy data from `other=` if non-None, otherwise metadata gets copied from the input.
+        """
+
+        self = cls.__new__(cls)
+        if isinstance(_hist, tuple(_histograms)):
+            self._from_histogram_cpp(_hist)  # type: ignore[arg-type]
+            if other is not None:
+                return cls._clone(self, other=other, memo=memo)
+            return self
+
+        assert isinstance(_hist, Histogram)
+
+        if other is None:
+            other = _hist
+
+        self._from_histogram_object(_hist)
+
+        if memo is NOTHING:
+            self.__dict__ = copy.copy(other.__dict__)
+        else:
+            self.__dict__ = copy.deepcopy(other.__dict__, memo)
+
+        for ax in self.axes:
+            if memo is NOTHING:
+                ax.__dict__ = copy.copy(ax._ax.metadata)
+            else:
+                ax.__dict__ = copy.deepcopy(ax._ax.metadata, memo)
+        return self
+
+    def _new_hist(self: H, _hist: CppHistogram, memo: Any = NOTHING) -> H:
+        """
+        Return a new histogram given a new _hist, copying current metadata.
+        """
+        return self.__class__._clone(_hist, other=self, memo=memo)
+
+    def _from_histogram_cpp(self, other: CppHistogram) -> None:
+        """
+        Import a Cpp histogram.
+        """
+        self._variance_known = True
+        self._hist = other
+        self.metadata = None
+        self.axes = self._generate_axes_()
+
     def _from_histogram_object(self, other: "Histogram") -> None:
         """
         Convert self into a new histogram object based on another, possibly
@@ -270,32 +344,12 @@ class Histogram:
 
         return AxesTuple(self._axis(i) for i in range(self.ndim))
 
-    def _new_hist(self: H, _hist: CppHistogram, memo: Any = NOTHING) -> H:
-        """
-        Return a new histogram given a new _hist, copying metadata.
-        """
-
-        other = self.__class__(_hist)
-        if memo is NOTHING:
-            other.__dict__ = copy.copy(self.__dict__)
-        else:
-            other.__dict__ = copy.deepcopy(self.__dict__, memo)
-        other.axes = other._generate_axes_()
-
-        for ax in other.axes:
-            if memo is NOTHING:
-                ax.__dict__ = copy.copy(ax._ax.metadata)
-            else:
-                ax.__dict__ = copy.deepcopy(ax._ax.metadata, memo)
-
-        return other
-
     @property
     def ndim(self) -> int:
         """
         Number of axes (dimensions) of the histogram.
         """
-        return self._hist.rank()  # type: ignore[no-any-return]
+        return self._hist.rank()
 
     def view(
         self, flow: bool = False
@@ -332,8 +386,8 @@ class Histogram:
 
         return self
 
-    def __radd__(
-        self: H, other: Union["Histogram", "np.typing.NDArray[Any]", float]
+    def __radd__(  # type: ignore[misc]
+        self: H, other: Union["np.typing.NDArray[Any]", float]
     ) -> H:
         return self + other
 
@@ -361,8 +415,8 @@ class Histogram:
         result = self.copy(deep=False)
         return result._compute_inplace_op("__imul__", other)
 
-    def __rmul__(
-        self: H, other: Union["Histogram", "np.typing.NDArray[Any]", float]
+    def __rmul__(  # type: ignore[misc]
+        self: H, other: Union["np.typing.NDArray[Any]", float]
     ) -> H:
         return self * other
 
@@ -449,6 +503,9 @@ class Histogram:
             available threads (usually two per core).
         """
 
+        if self._hist._storage_type is _core.storage.mean:
+            mean_storage_sample_check(sample)
+
         if (
             self._hist._storage_type
             not in {
@@ -465,32 +522,37 @@ class Histogram:
         weight_ars = _fill_cast(weight)
         sample_ars = _fill_cast(sample)
 
-        if threads is None or threads == 1:
-            self._hist.fill(*args_ars, weight=weight_ars, sample=sample_ars)
-            return self
-
         if threads == 0:
             threads = cpu_count()
 
+        if threads is None or threads == 1:
+            self._hist.fill(*args_ars, weight=weight_ars, sample=sample_ars)  # type: ignore[arg-type]
+            return self
+
         if self._hist._storage_type in {
             _core.storage.mean,
             _core.storage.weighted_mean,
         }:
             raise RuntimeError("Mean histograms do not support threaded filling")
 
-        data = [np.array_split(a, threads) for a in args_ars]  # type: ignore[no-untyped-call]
+        data: "List[Union[List[np.typing.NDArray[Any]], List[str]]]" = [
+            np.array_split(a, threads) if not isinstance(a, str) else [a] * threads  # type: ignore[arg-type, list-item]
+            for a in args_ars
+        ]
 
+        weights: "List[Any]"
         if weight is None or np.isscalar(weight):
             assert threads is not None
             weights = [weight_ars] * threads
         else:
-            weights = np.array_split(weight_ars, threads)  # type: ignore[no-untyped-call]
+            weights = np.array_split(weight_ars, threads)  # type: ignore[arg-type]
 
+        samples: "List[Any]"
         if sample_ars is None or np.isscalar(sample_ars):
             assert threads is not None
             samples = [sample_ars] * threads
         else:
-            samples = np.array_split(sample_ars, threads)  # type: ignore[no-untyped-call]
+            samples = np.array_split(sample_ars, threads)  # type: ignore[arg-type]
 
         if self._hist._storage_type is _core.storage.atomic_int64:
 
@@ -509,7 +571,7 @@ class Histogram:
                 sample: Optional[ArrayLike],
                 *args: "np.typing.NDArray[Any]",
             ) -> None:
-                local_hist = self._hist.__copy__()
+                local_hist = copy.copy(self._hist)
                 local_hist.reset()
                 local_hist.fill(*args, weight=weight, sample=sample)
                 with sum_lock:
@@ -548,8 +610,18 @@ class Histogram:
         """
         return cast(self, self._hist.axis(i), Axis)
 
+    @property
+    def storage_type(self) -> Type[Storage]:
+        return cast(self, self._hist._storage_type, Storage)  # type: ignore[return-value]
+
     @property
     def _storage_type(self) -> Type[Storage]:
+        if sys.version_info >= (3, 7):
+            warnings.warn(
+                "Accessing storage type has changed from _storage_type to storage_type, and will be removed in future.",
+                PendingDeprecationWarning,
+                stacklevel=2,
+            )
         return cast(self, self._hist._storage_type, Storage)  # type: ignore[return-value]
 
     def _reduce(self: H, *args: Any) -> H:
@@ -605,7 +677,7 @@ class Histogram:
         sep = "," if len(self.axes) > 0 else ""
         ret = f"{self.__class__.__name__}({first_newline}"
         ret += f",{newline}".join(repr(ax) for ax in self.axes)
-        ret += f"{sep}{storage_newline}storage={self._storage_type()}"  # pylint: disable=not-callable
+        ret += f"{sep}{storage_newline}storage={self.storage_type()}"  # pylint: disable=not-callable
         ret += ")"
         outer = self.sum(flow=True)
         if outer:
@@ -629,13 +701,13 @@ class Histogram:
         if callable(index):
             return index(self.axes[axis])
 
-        if isinstance(index, float):
+        if isinstance(index, float):  # type: ignore[unreachable]
             raise TypeError(f"Index {index} must be an integer, not float")
 
         if isinstance(index, SupportsIndex):
             if abs(int(index)) >= self._hist.axis(axis).size:
                 raise IndexError("histogram index is out of range")
-            return index % self._hist.axis(axis).size  # type: ignore[no-any-return]
+            return int(index) % self._hist.axis(axis).size
 
         return index
 
@@ -714,7 +786,7 @@ class Histogram:
         hist, *edges = self._hist.to_numpy(flow)
         hist = self.view(flow=flow) if view else self.values(flow=flow)
 
-        return (hist, edges) if dd else (hist, *edges)
+        return (hist, edges) if dd else (hist, *edges)  # type: ignore[return-value]
 
     def copy(self: H, *, deep: bool = True) -> H:
         """
@@ -727,7 +799,7 @@ class Histogram:
 
     def reset(self: H) -> H:
         """
-        Reset bin counters to default values.
+        Clear the bin counters.
         """
         self._hist.reset()
         return self
@@ -737,12 +809,15 @@ class Histogram:
         Check to see if the histogram has any non-default values.
         You can use flow=True to check flow bins too.
         """
-        return self._hist.empty(flow)  # type: ignore[no-any-return]
+        return self._hist.empty(flow)
 
     def sum(self, flow: bool = False) -> Union[float, Accumulator]:
         """
         Compute the sum over the histogram bins (optionally including the flow bins).
         """
+        if any(x == 0 for x in (self.axes.extent if flow else self.axes.size)):
+            return self.storage_type.accumulator()
+
         return self._hist.sum(flow)  # type: ignore[no-any-return]
 
     @property
@@ -750,7 +825,7 @@ class Histogram:
         """
         Total number of bins in the histogram (including underflow/overflow).
         """
-        return self._hist.size()  # type: ignore[no-any-return]
+        return self._hist.size()
 
     @property
     def shape(self) -> Tuple[int, ...]:
@@ -771,7 +846,7 @@ class Histogram:
         if not hasattr(indexes, "items") and all(
             isinstance(a, SupportsIndex) for a in indexes
         ):
-            return self._hist.at(*indexes)  # type: ignore[no-any-return]
+            return self._hist.at(*indexes)  # type: ignore[no-any-return, arg-type]
 
         integrations: Set[int] = set()
         slices: List[_core.algorithm.reduce_command] = []
@@ -852,6 +927,9 @@ class Histogram:
             new_reduced.view(flow=True)[...] = reduced.view(flow=True)[tuple_slice]
             reduced = new_reduced
             integrations = {i - sum(j <= i for j in pick_each) for i in integrations}
+            pick_set = {
+                i - sum(j <= i for j in pick_each): v for i, v in pick_set.items()
+            }
             for slice_ in slices:
                 slice_.iaxis -= sum(j <= slice_.iaxis for j in pick_each)
 
@@ -861,7 +939,8 @@ class Histogram:
 
         if pick_set:
             warnings.warn(
-                "List indexing selection is experimental. Removed bins are not placed in overflow."
+                "List indexing selection is experimental. Removed bins are not placed in overflow.",
+                stacklevel=2,
             )
             logger.debug("Slices for picking sets: %s", pick_set)
             axes = [reduced.axis(i) for i in range(reduced.rank())]
@@ -870,14 +949,13 @@ class Histogram:
                 selection = copy.copy(pick_set[i])
                 ax = reduced.axis(i)
                 if ax.traits_ordered:
-                    raise RuntimeError(
-                        f"Axis {i} is not a categorical axis, cannot pick with list"
-                    )
+                    msg = f"Axis {i} is not a categorical axis, cannot pick with list: {ax}"
+                    raise RuntimeError(msg)
 
                 if ax.traits_overflow and ax.size not in pick_set[i]:
                     selection.append(ax.size)
 
-                new_axis = axes[i].__class__([axes[i].value(j) for j in pick_set[i]])
+                new_axis = axes[i].__class__([axes[i].value(j) for j in pick_set[i]])  # type: ignore[call-arg]
                 new_axis.metadata = axes[i].metadata
                 axes[i] = new_axis
                 reduced_view = np.take(reduced_view, selection, axis=i)
@@ -930,7 +1008,7 @@ class Histogram:
         if (
             value.ndim > 0
             and len(view.dtype) > 0  # type: ignore[arg-type]
-            and len(value.dtype) == 0
+            and len(value.dtype) == 0  # type: ignore[arg-type]
             and len(view.dtype) == value.shape[-1]  # type: ignore[arg-type]
         ):
             value_shape = value.shape[:-1]
@@ -1076,7 +1154,7 @@ class Histogram:
 
         if hasattr(view, "sum_of_weights"):
             valid = view.sum_of_weights**2 > view.sum_of_weights_squared  # type: ignore[union-attr]
-            return np.divide(  # type: ignore[no-any-return]
+            return np.divide(
                 view.variance,  # type: ignore[union-attr]
                 view.sum_of_weights,  # type: ignore[union-attr]
                 out=np.full(view.sum_of_weights.shape, np.nan),  # type: ignore[union-attr]
@@ -1084,7 +1162,7 @@ class Histogram:
             )
 
         if hasattr(view, "count"):
-            return np.divide(  # type: ignore[no-any-return]
+            return np.divide(
                 view.variance,  # type: ignore[union-attr]
                 view.count,  # type: ignore[union-attr]
                 out=np.full(view.count.shape, np.nan),  # type: ignore[union-attr]
@@ -1122,7 +1200,7 @@ class Histogram:
             return view
 
         if hasattr(view, "sum_of_weights"):
-            return np.divide(  # type: ignore[no-any-return]
+            return np.divide(
                 view.sum_of_weights**2,  # type: ignore[union-attr]
                 view.sum_of_weights_squared,  # type: ignore[union-attr]
                 out=np.zeros_like(view.sum_of_weights, dtype=np.float64),  # type: ignore[union-attr]
